@@ -45,6 +45,8 @@
 #' Default is \code{FALSE}. If \code{TRUE}, the residual (weighted) sum-of-squares,
 #' the squared (Euclidean) norm of the current parameter estimates and the condition number of the Jacobian
 #' are printed after each iteration.
+#' @param weights an optional numeric vector of (fixed) weights of length \code{n}. When present, 
+#' the objective function is weighted least squares. 
 #' @return
 #' If \code{fn} is a \code{formula} returns a list object of class \code{nls}.
 #' If \code{fn} is a \code{function} returns a list object of class \code{gsl_nls}.
@@ -127,8 +129,8 @@ gsl_nls_large <- function (fn, ...) {
 #' If \code{fn} is a \code{formula}, the returned list object is of classes \code{gsl_nls} and \code{nls}.
 #' Therefore, all generic functions applicable to objects of class \code{nls}, such as \code{anova}, \code{coef}, \code{confint},
 #' \code{deviance}, \code{df.residual}, \code{fitted}, \code{formula}, \code{logLik}, \code{nobs}, \code{predict}, \code{print}, \code{profile},
-#' \code{residuals}, \code{summary}, \code{vcov} and \code{weights} are also applicable to the returned list object.
-#' In addition, a method \code{confintd} is available for inference of derived parameters.
+#' \code{residuals}, \code{summary}, \code{vcov}, \code{hatvalues}, \code{cooks.distance} and \code{weights} are also applicable to the returned
+#' list object. In addition, a method \code{confintd} is available for inference of derived parameters.
 #' @export
 gsl_nls_large.formula <- function(fn, data = parent.frame(), start,
                                   algorithm = c("lm", "lmaccel", "dogleg", "ddogleg", "subspace2D", "cgst"),
@@ -253,8 +255,8 @@ gsl_nls_large.formula <- function(fn, data = parent.frame(), start,
       mf <- as.list(mf)
       wts <- if (!mWeights) model.weights(mf) else NULL
     }
-    if (!is.null(wts) && any(wts < 0 | is.na(wts)))
-      stop("missing or negative weights not allowed")
+    if (!is.null(wts) && any(wts <= 0 | is.na(wts)))
+      stop("missing or non-positive weights not allowed")
   }
   else {
     stop("no data variables present")
@@ -409,7 +411,7 @@ gsl_nls_large.formula <- function(fn, data = parent.frame(), start,
   cFit <- .Call(C_nls_large, .fn, .lhs, .jac, .fvv, environment(), start, wts, .ctrl_int, .ctrl_dbl, PACKAGE = "gslnls")
 
   ## convert to nls object
-  m <- nlsModel(formula, mf, cFit$par, wts, jac)
+  m <- nlsModel(formula, mf, cFit, wts, jac)
 
   convInfo <- list(
     isConv = as.logical(!cFit$conv),
@@ -451,8 +453,8 @@ gsl_nls_large.formula <- function(fn, data = parent.frame(), start,
 #' Although the returned object is not of class \code{nls}, the following generic functions remain
 #' applicable for an object of class \code{gsl_nls}: \code{anova}, \code{coef}, \code{confint}, \code{deviance},
 #' \code{df.residual}, \code{fitted}, \code{formula}, \code{logLik}, \code{nobs}, \code{predict}, \code{print},
-#' \code{residuals}, \code{summary}, \code{vcov} and \code{weights}. In addition, a method \code{confintd}
-#' is available for inference of derived parameters.
+#' \code{residuals}, \code{summary}, \code{vcov}, \code{hatvalues}, \code{cooks.distance} and \code{weights}.
+#' In addition, a method \code{confintd} is available for inference of derived parameters.
 #' @export
 gsl_nls_large.function <- function(fn, y, start,
                                    algorithm = c("lm", "lmaccel", "dogleg", "ddogleg", "subspace2D", "cgst"),
@@ -586,8 +588,8 @@ gsl_nls_large.function <- function(fn, y, start,
   if(!missing(weights)) {
     if(!is.numeric(weights) || !identical(length(weights), length(y)))
       stop("'weights' should be numeric equal in length to 'y'")
-    if (any(weights < 0 | is.na(weights)))
-      stop("missing or negative weights not allowed")
+    if (any(weights <= 0 | is.na(weights)))
+      stop("missing or non-positive weights not allowed")
   } else {
     weights <- NULL
   }
